package cz.mendelu.dnaAnalyser.batch;

import cz.mendelu.dnaAnalyser.utils.controller.Response;
import cz.mendelu.dnaAnalyser.utils.controller.ResponseList;
import cz.mendelu.dnaAnalyser.utils.controller.ResponsePage;
import cz.mendelu.dnaAnalyser.utils.model.Identifiable;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.MessageFormat;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

@Slf4j
@Aspect
@Component
public class BatchFilter {

    @Autowired
    private BatchService batchService;

    @AfterReturning(
            pointcut="bean(*Controller)",
            returning="retVal")
    public void doBatchCheck(Object retVal) {
        if (retVal instanceof Response) {
            Response response = (Response) retVal;
            if (response.getPayload() instanceof Identifiable) {
                Identifiable payload = (Identifiable) response.getPayload();
                batchService.findBatchFor(payload)
                        .ifPresent(response::setBatch);
            }
            //log.info("Batch filter for entity {} batch {}.", response.getPayload(), batch);
        } else if (retVal instanceof ResponsePage) {
            ResponsePage page = (ResponsePage) retVal;
            Map<UUID, Batch> batches = findBatches(page.getItems());
            page.setBatches(batches);
            //log.info("Batch filter for page {} batches {}.", page, batches);
        } else if (retVal instanceof ResponseList) {
            ResponseList list = (ResponseList) retVal;
            Map<UUID, Batch> batches = findBatches(list.getItems());
            list.setBatches(batches);
            //log.info("Batch filter for list {} batches {}.", list, batches);
        }
    }

    private Map<UUID, Batch> findBatches(List items) {
        Map<UUID, Batch> batches = new LinkedHashMap<>();
        for (Object item: items) {
            if (!(item instanceof Identifiable)) {
                return null;
            }
            Identifiable identifiable = (Identifiable) item;
            try {
                batchService.findBatchFor(identifiable)
                        .ifPresent(batch -> batches.put(identifiable.getId(), batch));
            } catch (RuntimeException e) {
                log.warn(MessageFormat.format("Append batch for entity {1} id: {2} failed.", identifiable.getClass(), identifiable.getId()), e);
            }

        }
        return batches;
    }
}
