package cz.mendelu.dnaAnalyser.overlay;

import cz.mendelu.dnaAnalyser.analyse.g4hunter.quadruplex.AggregateQuadruplexMapper;
import cz.mendelu.dnaAnalyser.sequence.annotation.feature.Feature;
import cz.mendelu.dnaAnalyser.utils.analyse.data.DataLocalisedMapper;
import cz.mendelu.dnaAnalyser.utils.analyse.data.Localised;
import cz.mendelu.dnaAnalyser.utils.analyse.model.Analyse;
import cz.mendelu.dnaAnalyser.utils.analyse.service.DataSessionService;
import cz.mendelu.dnaAnalyser.utils.analyse.service.heatmap.Heatmap;
import cz.mendelu.dnaAnalyser.utils.analyse.service.heatmap.Segment;
import org.apache.ibatis.session.RowBounds;
import org.apache.ibatis.session.SqlSession;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.IntStream;

@Service
public interface LocalisedOverlay extends DataSessionService {

    default <T extends DataLocalisedMapper> Map<String, OverlayItem> annotationOverlay(Class<T> mapperClass, Feature annotation, SqlSession session, Map<String, OverlayItem> output, String groupBy, String labelGroupBy, List<Double> intervals, List<String> groupIntervals, Integer from, Integer to) {

            DataLocalisedMapper mapper = session.getMapper(mapperClass);
            List<Localised> analyseList = mapper.getAllBetweenPositionOrderAsc(from, to, "POSITION", new RowBounds());

            double coverage = 0.0;
             Integer iteration = 0;
            OverlayItem item = output.get(getOrderField(annotation, labelGroupBy));
            if(item == null){
                OverlayItem overlay = new OverlayItem();
                Map<String, Object> group = new HashMap<>();
                overlay.setOccureneces(group);
                overlay.setAverageCoverage(0.0);
                overlay.setAverageLength(0.0);
                overlay.setTotalLength(0.0);
                output.put(getOrderField(annotation, labelGroupBy).toString(),overlay); // featrue type je label
            }
        //if ((annotation.getBefore() - analyse.getPosition()) >= 100000) analyseList.remove(analyse);  TODO

            for (Localised localisedAnalyse : analyseList) {
                AnalyseResult analyse = new AnalyseResult(localisedAnalyse.getPosition(), localisedAnalyse.getLength());
                analyse.setGroupBy(getOrderField(localisedAnalyse, groupBy));
                String groupId = intervals == null ? getGroupId(analyse, output, annotation.getFeatureType()) : getGroupIdByInterval(analyse, groupIntervals);

                if (localisedAnalyse.getPosition() > annotation.getAfter()) {
                    break;
                } else {
                    Map<String, Object> group = item.getOccureneces();
                    EnumMap<Occurence, Integer> occurence = (EnumMap<Occurence, Integer>) group.get(groupId);
                    if (annotation.isBefore(analyse)) {
                        updateOccurence(Occurence.BEFORE, occurence, coverage, iteration, annotation, analyse);
                    }
                    if (annotation.isIn(analyse)) {
                        updateOccurence(Occurence.IN, occurence, coverage, iteration, annotation, analyse);
                    }
                    if (annotation.isAfter(analyse)) {
                        updateOccurence(Occurence.AFTER, occurence, coverage, iteration, annotation, analyse);
                    }
                }
            }
            computeLength(item, from, to);
            computeAverageLength(item, to - from);
            item.setAverageCoverage(coverage);

        return output;


    }

    static void updateOccurence(Occurence state, EnumMap<Occurence, Integer> occurence, double coverage, Integer iteration, Feature annotation, AnalyseResult analyse){
        occurence.put(state, occurence.get(state) + 1);
        computeCoverage(coverage, iteration, annotation, analyse);
    }

    static String getGroupIdByInterval(AnalyseResult analyse, List<String> intervals) {
        for (int i = 0; i < intervals.size(); i++) {
            String interval = intervals.get(i);
            String[] values = interval.split("-");
            Float comp = Float.parseFloat(analyse.getGroupBy().toString());
            if (Float.parseFloat(values[0]) < comp && comp <= Float.parseFloat(values[1])) {
                return interval;
            }
        }
        return intervals.get(0);

    }

    static String getGroupId(AnalyseResult analyse, Map<String, OverlayItem> res, String featureType) { //featureType je label
        OverlayItem item = res.get(featureType);
        Map<String, Object> group = item.getOccureneces();
        String groupBy = analyse.getGroupBy().toString();
        if (!group.containsKey(groupBy)) {
            EnumMap<Occurence, Integer> occurenceMap = new EnumMap<>(Occurence.class);
            occurenceMap.put(Occurence.BEFORE, 0);
            occurenceMap.put(Occurence.IN, 0);
            occurenceMap.put(Occurence.AFTER, 0);
            group.put(groupBy, occurenceMap);
        }
        return groupBy;
    }

    static void computeCoverage(Double coverage, Integer iteration, Feature feature, AnalyseResult analyseResult) {
        iteration++;
        int l = Math.max(Integer.parseInt(feature.getStartPosition()), analyseResult.getPosition());
        int r = Math.min(Integer.parseInt(feature.getEndPosition()), analyseResult.getPosition() + analyseResult.getLength());
        double result = (double) (r - l) / (Double.parseDouble(feature.getEndPosition()) - Double.parseDouble(feature.getStartPosition()));

        coverage = (coverage + result) / iteration;

    }

    static void computeLength(OverlayItem item, int position, int length) {
        if (item.getStart() > position){
            item.setStart(position);
        }
        if (item.getStop() < position + length){
            item.setStop(position + length);
        }
        item.setTotalLength(item.getStop() - item.getStart());
    }

    static void computeAverageLength(OverlayItem item, int length) {
        if (item.getAverageLength() == 0){
            item.setAverageLength(length);
        }else{
            item.setAverageLength((item.getAverageLength() + length) /2);
        }
    }

    static Object getOrderField(Object o, String groupBy) {
        Class<?> clazz = o.getClass();

        for (Field field : clazz.getDeclaredFields()) {
            String name = field.getName();
            field.setAccessible(true);
            Object value = null;
            try {
                value = field.get(o);
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
            if (name.equals(groupBy)) {
                return value;
            }
        }
        throw new IllegalArgumentException("Target analyse does not contain group by field " + groupBy);
    }


}
