package cz.mendelu.dnaAnalyser.batch

import cz.mendelu.dnaAnalyser.batch.BatchFilter
import cz.mendelu.dnaAnalyser.utils.controller.Response
import cz.mendelu.dnaAnalyser.utils.controller.ResponsePage
import cz.mendelu.dnaAnalyser.utils.model.MockIdentifiable
import cz.mendelu.dnaAnalyser.batch.Batch
import cz.mendelu.dnaAnalyser.batch.BatchService
import spock.lang.Specification

class BatchFilterTest extends Specification {

    BatchFilter batchFilter

    BatchService batchService

    def setup() {
        batchFilter = new BatchFilter()
        batchFilter.batchService = batchService = Mock(BatchService)
    }

    def "test process not-batch response"() {
        given:
        def retVal = 'any type'

        when:
        batchFilter.doBatchCheck(retVal)

        then:
        retVal == 'any type'
        0 * batchService._
    }

    def "test process response"() {
        given:
        def entity = new MockIdentifiable(id: UUID.randomUUID())
        def response = new Response(payload: entity)
        def batch = new Batch()

        when:
        batchFilter.doBatchCheck(response)

        then:
        System.out.println(response)
        1 * batchService.findBatchFor(entity) >> Optional.of(batch)
        response.getBatch() != null
    }

    def "test process page"() {
        given:
        def entities = [new MockIdentifiable(id: UUID.randomUUID()), new MockIdentifiable(id: UUID.randomUUID())]
        def response = new ResponsePage(items: entities)
        def batches = [Optional.of(new Batch()), Optional.of(new Batch())]

        when:
        batchFilter.doBatchCheck(response)

        then:

        2 * batchService.findBatchFor(_) >>> batches
        response.getBatches() != null
        response.getBatches().size() == 2
    }

    def "test process list"() {
        given:
        def entities = [new MockIdentifiable(id: UUID.randomUUID()), new MockIdentifiable(id: UUID.randomUUID())]
        def response = new ResponsePage(items: entities)
        def batches = [Optional.of(new Batch()), Optional.of(new Batch())]

        when:
        batchFilter.doBatchCheck(response)

        then:
        2 * batchService.findBatchFor(_) >>> batches
        response.getBatches() != null
        response.getBatches().size() == 2
    }

    def "test process page with null batch"() {
        given:
        def entities = [new MockIdentifiable(id: UUID.randomUUID()), new MockIdentifiable(id: UUID.randomUUID())]
        def response = new ResponsePage(items: entities)
        def batches = [null, Optional.of(new Batch())]

        when:
        batchFilter.doBatchCheck(response)

        then:
        2 * batchService.findBatchFor(_) >>> batches
        response.getBatches() != null
        response.getBatches().size() == 1
    }

    def "test process list with null batch"() {
        given:
        def entities = [new MockIdentifiable(id: UUID.randomUUID()), new MockIdentifiable(id: UUID.randomUUID())]
        def response = new ResponsePage(items: entities)
        def batches = [null, Optional.of(new Batch())]

        when:
        batchFilter.doBatchCheck(response)

        then:
        2 * batchService.findBatchFor(_) >>> batches
        response.getBatches() != null
        response.getBatches().size() == 1
    }
}
