package cz.mendelu.dnaAnalyser.user

import cz.mendelu.dnaAnalyser.utils.api.ApiRestTemplate
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpStatus
import org.springframework.test.context.ActiveProfiles
import spock.lang.Ignore
import spock.lang.Specification

@Ignore
@SpringBootTest(webEnvironment = RANDOM_PORT)
@ActiveProfiles("test")
class UserControllerTest extends Specification {

    @Autowired
    ApiRestTemplate api

    def setup() {
        api.authentication("user@mendelu.cz", "user")
    }

    def "GET:/api/user Get current user"() {
        when:
        def response = api.get('/api/user')

        then:
        response.status == HttpStatus.OK.value()
    }

    def "GET:/api/user admin user"() {
        setup:
        api.authentication("admin@mendelu.cz", "admin")

        when:
        def response = api.get('/api/user')

        then:
        response.status == HttpStatus.OK.value()
    }

    def "POST:/api/user Register new user"() {
        when:
        def response = api.post('/api/user', new UserController.Register(login: "register@UserControllerTest.cz", password: "test"))

        then:
        response.status == HttpStatus.CREATED.value()

        and:
        def user = response.body.payload
        user.id != null
        user.username == 'register@UserControllerTest.cz'
        user.enabled == true
        user.lastPasswordResetDate != null
        user.createdDate != null
        user.authorities == ['ROLE_USER']
    }

    def "GET:/api/user Get current user by JWT token"() {
        when:
        def response = api.get("/api/user")

        then:
        response.status == HttpStatus.OK.value()

        and:
        def user = response.body.payload
        user.id != null
        user.username == 'user@mendelu.cz'
        user.enabled == true
        user.authorities == ['ROLE_USER']
    }

    def "DELETE:/api/user Delete user, current user is defined by JWT token"() {
        given:
        api.post('/api/user', new UserController.Register(login: 'delete@UserControllerTest.cz', password: "test"))
        api.authentication('delete@UserControllerTest.cz', "test")

        when:
        def response = api.delete("/api/user")

        then:
        response.status == HttpStatus.NO_CONTENT.value()
    }

    def "PUT:/api/user/password Change user password for current user by JWT token"() {
        given:
        api.post('/api/user', new UserController.Register(login: 'change@UserControllerTest.cz', password: "test"))
        api.authentication('change@UserControllerTest.cz', "test")

        when:
        def response = api.put("/api/user/change-password", new UserController.ChangePassword(oldPassword: 'test', newPassword: 'change'))

        then:
        response.status == HttpStatus.ACCEPTED.value()
    }

    def "POST:/api/user/password Reset user password"() {
        given:
        api.post('/api/user', new UserController.Register(login: "reset@UserControllerTest.cz", password: "test"))

        when:
        def response = api.put("/api/user/reset-password", new UserController.ResetPassword(login: 'reset@UserControllerTest.cz'))

        then:
        response.status == HttpStatus.NO_CONTENT.value()
    }

}

import static org.springframework.boot.test.context.SpringBootTest.WebEnvironment.RANDOM_PORT
